static char rcsid[] = "$Id: dbgobj.c,v 1.4 1997/02/28 05:24:11 ngoddard Exp $";

/*
** $Log: dbgobj.c,v $
** Revision 1.4  1997/02/28 05:24:11  ngoddard
** support for lookahead optimisation, bug fixes
**
** Revision 1.3  1996/06/21 17:32:59  ngoddard
** new Msg structure
**
 * Revision 1.2  1995/06/09  19:02:09  ngoddard
 * various fixes for ANSI C, function prototypes, and Solaris
 *
 * Revision 1.1  1995/04/03  21:51:14  ngoddard
 * Initial revision
 *
*/

#include "dbg_ext.h"
#include "dbg_g@.h"
#include "assert.h"

/*
** An object for testing the parallel library.
** For now it simply sets its value to the sum of the incoming values
** 
*/

#define MSGACTIVE(M) ((M)->type < 0)

DbgFunc(dbg,action)
     register struct dbg_type *dbg;
     Action *action;
{
  MsgIn	*msg;
  MsgOut * msgout;
  int i;
  double time = SimulationTime();

  SELECT_ACTION(action){
  case PROCESS:
    dbg->value = 0;
    MSGLOOP(dbg,msg) {
    case dbg_ACTIVE:
    case dbg_ACTIVEL:
      if (dbg->evtime >= 0 && dbg->evtime <= SimulationTime())
	{
	  if (IsSilent() < 3)
	    fprintf(stdout, "dbg object processed event from time %f at time %f\n",
		    dbg->evtime-dbg->delay, SimulationTime());
	  dbg->evtime = -1;	/* so we can receive another */
	}
      break;
    default:
      for (i = 0; i < ((int) msg->nslots); i++)
	dbg->value += MSGVALUE(msg,i);
    }
    /* send EVENTS */
    if (dbg->value > dbg->init_value)
      MSGOUTLOOP(dbg, msgout) {
	if (MSGACTIVE(msgout))
	  CallEventAction(dbg, msgout, time);
      }
    break;
  case EVENT:
    {
      MsgIn * msg = (MsgIn*) action->data;
      double * time = (double *) action->argv;
      double tim = *time;
      switch (msg->type)
	{
	case dbg_ACTIVE:
	  if (IsSilent() < 3)
	    printf("event (from time=%g) caught in dbg object (at time=%g)\n",
		   *time, SimulationTime());
	  break;
	case dbg_ACTIVEL:
	  assert(dbg->delay > 0.);
	  tim += dbg->delay;	/* time to process */
	  if (tim < SimulationTime())
	    fprintf(stdout,
		    "***ERROR, activel msg delivered (%f seconds) late!\n",
		    SimulationTime() - tim);
	  else
	    {
	      if (IsSilent() < 2)
		fprintf(stdout, "activel msg (time=%g) arrived at time=%g\n",
			*time, SimulationTime());
	      if (dbg->evtime < 0)
		dbg->evtime = tim;
	      else
		fprintf(stdout, "ignored, active msg already in pipeline\n");
	    }
	  break;
	default:
	  printf("unknown event type %d (from time=%g) caught in dbg object (at time=%g)\n",
		 (int) (msg->type), tim, SimulationTime());
	}
      }
    break;
  case RESET:
    dbg->value = dbg->init_value;
    dbg->evtime = -1.;
    dbg->activel_exists = 0;
    break;
  case CREATE:
    dbg->value = dbg->init_value = dbg->delay = 0;
    dbg->evtime = -1.;
    dbg->activel_exists = 0;
    dbg->weight = 1.;
    break;
  case DBGDISPLAY:
    fprintf(stdout, "Dbg val: %d ival: %d\n", dbg->value, dbg->init_value);
    /* show messages? */
    break;
  case ADDMSGIN:
    msg = (Msg *) (action->data);
    switch (msg->type)
      {
      case dbg_ACTIVE:
	break;
      case dbg_ACTIVEL:
	assert(!dbg->activel_exists);
	dbg->activel_exists = 1;
	break;
      default:
	printf("unknown event type %d added to dbg_object \n",
	       (int) (msg->type));
	break;
      }
    break;
  }
}

